#version 330 core

in vec2 UV;
in vec3 positionWorld;

in vec3 eyeDirCam;
in vec3 lightDirCam;
in vec3 eyeDirTangent;
in vec3 lightDirTangent;

out vec4 outColor[2];

uniform sampler2D DiffuseTextureSampler;
uniform sampler2D NormalTextureSampler;
uniform mat4 V;
uniform mat4 M;
uniform mat4 P;
uniform vec3 lightPosWorld;

//Shadow:
in vec4 shadowCoord;
uniform sampler2DShadow shadowMap;

vec2 poissonDisk[16] = vec2[](
   vec2( -0.94201624, -0.39906216 ),
   vec2( 0.94558609, -0.76890725 ),
   vec2( -0.094184101, -0.92938870 ),
   vec2( 0.34495938, 0.29387760 ),
   vec2( -0.91588581, 0.45771432 ),
   vec2( -0.81544232, -0.87912464 ),
   vec2( -0.38277543, 0.27676845 ),
   vec2( 0.97484398, 0.75648379 ),
   vec2( 0.44323325, -0.97511554 ),
   vec2( 0.53742981, -0.47373420 ),
   vec2( -0.26496911, -0.41893023 ),
   vec2( 0.79197514, 0.19090188 ),
   vec2( -0.24188840, 0.99706507 ),
   vec2( -0.81409955, 0.91437590 ),
   vec2( 0.19984126, 0.78641367 ),
   vec2( 0.14383161, -0.14100790 )
);

float random(vec3 seed, int i){
	vec4 seed4 = vec4(seed,i);
	float dot_product = dot(seed4, vec4(12.9898,78.233,45.164,94.673));
	return fract(sin(dot_product) * 43758.5453);
}

void main(){

	vec3 lightColor = vec3(1,1,1);
	float lightPower = 10.0;

	vec3 diffuseColor = texture(DiffuseTextureSampler, UV).rgb;
	vec3 ambientColor = vec3(0.4,0.4,0.4) * diffuseColor;
	vec3 specularColor = vec3(1,1,1);

	vec3 texNomal_tangent = normalize(texture(NormalTextureSampler, vec2(UV.x, -UV.y) ).rgb * 2.0 - 1.0);

	float distance = length(lightPosWorld - positionWorld);
	vec3 n = texNomal_tangent;
	vec3 l = normalize(lightDirTangent);
	float cosTheta = clamp(dot(n, l), 0, 1);
	
	vec3 e = normalize(eyeDirTangent);
	vec3 r = reflect(-l, n);
	float cosAlpha = clamp(dot(e, r), 0, 1);
	cosAlpha = cosAlpha * cosTheta;

	//Shadow:
	float visibility = 1.0;

	float bias = 0.005 * tan(acos(cosTheta));
	bias = clamp(bias, 0, 0.01);

	for (int i=0; i<4; i++){
		//int index = i;
		//int index = int(16.0 * random(floor(positionWorld.xyz * 1000.0), i)) % 16;
		int index = int(16.0*random(gl_FragCoord.xyy, i))%16;
		visibility -= 0.16 * (1.0 - texture(shadowMap, vec3(shadowCoord.xy + poissonDisk[index] / 700.0,  (shadowCoord.z - bias) / shadowCoord.w)));
	}

	outColor[0] = vec4(ambientColor + diffuseColor * lightColor * lightPower * cosTheta / distance
					+ specularColor * lightColor * lightPower * pow(cosAlpha, 5) / (distance * distance), 1.0);
	outColor[1] = vec4(0.0,0.0,0.0,0.0);
}
