#version 330 core

in vec3 position;
in vec2 uv;
in vec3 normal;
in vec3 tangent;
in vec3 bitangent;

out vec2 UV;
out vec3 positionWorld;
out vec3 eyeDirTangent;
out vec3 lightDirTangent;
out vec3 eyeDirCam;
out vec3 lightDirCam;

uniform mat4 V;
uniform mat4 M;
uniform mat4 P;
uniform vec3 lightPosWorld;

//ShadowMapping:
out vec4 shadowCoord;
uniform mat4 depthBiasVP;

void main(){

	//Shadow
	mat4 depthBiasMVP = depthBiasVP * M;
	shadowCoord = depthBiasMVP * vec4(position,1);

	mat4 MVP = P * V * M;
	gl_Position =  MVP * vec4(position,1);
	UV = uv;

	mat4 VM = V * M;
	mat3 MV3x3 = mat3( VM[0].xyz, VM[1].xyz, VM[2].xyz);
	
	positionWorld = (M * vec4(position, 1)).xyz;
	
	vec3 vertexPosCam = (V * M * vec4(position, 1)).xyz;
	eyeDirCam = vec3(0,0,0) - vertexPosCam;
	vec3 lightPosCam = ( V * vec4(lightPosWorld,1)).xyz;
	lightDirCam = lightPosCam + eyeDirCam;
	
	vec3 vTangent_cam = MV3x3 * tangent;
	vec3 vBitangent_cam = MV3x3 * bitangent;
	vec3 vNormal_cam = MV3x3 * normal;
	
	mat3 tbn = transpose(mat3(vTangent_cam, vBitangent_cam, vNormal_cam));

	lightDirTangent = tbn * lightDirCam;
	eyeDirTangent = tbn * eyeDirCam;
}

