#version 330 core

in vec3 position;
in vec2 uv;
in vec3 normal;
in vec3 tangent;
in vec3 bitangent;

out vec2 UV;
out vec3 positionWorld;

out vec3 lightPosTangent;
out vec3 viewPosTangent;
out vec3 vertexPosTangent;

uniform mat4 V;
uniform mat4 M;
uniform mat4 P;
uniform vec3 lightPosWorld;

//ShadowMapping:
out vec4 shadowCoord;
uniform mat4 depthBiasVP;

void main(){

	//Shadow
	mat4 depthBiasMVP = depthBiasVP * M;
	shadowCoord = depthBiasMVP * vec4(position,1);

	mat4 MVP = P * V * M;
	gl_Position =  MVP * vec4(position,1);
	UV = uv;

	mat4 VM = V * M;
	
	positionWorld = (M * vec4(position, 1)).xyz;
	
	vec3 vertexPosCam = (V * M * vec4(position, 1)).xyz;
	
	mat3 normalMatrix = transpose(inverse(mat3(M)));
    vec3 T = normalize(normalMatrix * tangent);
    vec3 B = normalize(normalMatrix * bitangent);
    vec3 N = normalize(normalMatrix * normal);  
	
	mat3 TBN = transpose(mat3(T, B, N));  
    lightPosTangent = TBN * lightPosWorld;
    viewPosTangent  = TBN * vertexPosCam;
    vertexPosTangent  = TBN * positionWorld;
	
}

